// Copyright (C) 2006-2013 Xilinx, Inc.  All rights reserved
// $Id: CadFile.h,v 1.29.206.3 2013/05/10 00:04:05 jcorbett Exp $
#ifndef Iv_CadFile_h
#define Iv_CadFile_h

#include <vector>
#include <string>

#include "XOS/Portability/export/Port_Type.h"
#include "XOS/Portability/export/Port_FileName.h"
#ifdef _MSC_VER
   #pragma warning(push)
   #pragma warning(disable:4251)
#endif
#include "XCad/export/XCad_RMed.h"
#ifdef _MSC_VER
   #pragma warning(pop)
#endif
#include "Databases/Rf/export/Facade/Rf_Cell.h"

#include "TUtilities/export/smart_ptr.hpp"
#include "TUtilities/export/regex.hpp"

#include "DeviceModel.h"

// R#accept_isolation_regions

// Class to manage XCad and Pds (physical design) data from an NCD file.
//
// The loader is held because the view is invalidated when it is destroyed.
// The rmed is held because it has a funky allocator and deallocator.
//
// This class is in transition from the old multiple NCD approach to
// the new single NCD approach. The new behavior is that the parameter
// to the constructor is a module name. Nets will be obtained from the
// combined design under the module with the given name.
//
// If the module parameter has file extension .ncd, then it will be
// assumed to be a path to an NCD file as before. However, the module
// name will be extracted from that NCD file so that the new method
// can be compared to the old.
class CadFile {
public:
   typedef std::vector<XCad_CadNetwork*> NetVec;
   typedef NetVec::iterator NetVecIt;
public:
   CadFile(const std::string& ncdPath);
   CadFile(const std::string& ncdOrModule,
           boost::shared_ptr<CadFile>&,
           const std::vector<XCad_CadBlock*>& isolatedModules);
   ~CadFile();
   void SetDeviceModel(DeviceModel* deviceModel);
   DeviceModel* GetDeviceModel();
   const std::string GetFileName();
   const std::string GetModuleName();
   XCad_CadBlock* GetModule();
   XCad_CadBlock* GetBlockByUserHierName(const std::string& name);
   XCad_CadView* GetView();
   XCad_RMed* GetRMed();
   NetVec& GetNets();
   DeviceModel::SiteRectVec& GetSiteRectList();
   bool Contains(Port::UINT32 column, Port::UINT32 row);
   bool Contains(Port::UINT32 column, Port::UINT32 row,
                 const std::string& siteType);

private:
   void ReportInvalidModule(std::vector<XCad_CadBlock*>& blocks,
                            const std::string& ncdOrModule,
                            boost::shared_ptr<CadFile>&);
   bool HasNet(const XCad_CadNetwork* net,
               const XCad_CadBlock* module) const;
   bool IsIsolatedNet(const std::vector<XCad_CadBlock*>& isolatedModules,
                      const XCad_CadNetwork* net) const;
   void LoadNcd(const std::string& ncdPath);
   std::pair<const std::string, const std::string> FindNprSiteRangeListText();
   void FindSiteRangeString();
   void ParseRangeList(const std::string& areaGroup,
                       const std::string& rangeListString);
   bool ParseOneRangeList(const std::string& areaGroup,
                          const std::string& rangeListString);
   void Finalize();

private:
   Port::FileName ncdPath;
   XCad_CadBlock* module;
   Pds_PahDesignLoaderT<XCad_CadTraits>* loader;
   DeviceModel* deviceModel;
   bool isDeviceModelOwner;
   XCad_CadView* view;
   XCad_RMed* rmed;
   NetVec nets;
   DeviceModel::SiteRectVec siteRectList;
};

#endif
