// Copyright (C) 2006-2013 Xilinx, Inc.  All rights reserved
// $Id: Data.h,v 1.57.206.2 2013/06/04 23:11:30 jcorbett Exp $
#ifndef Iv_Data_h
#define Iv_Data_h

#include <utility>
#include <vector>
#include <string>
#include <cstring>
#include <map>
#include <set>

#include "TUtilities/export/smart_ptr.hpp"

#include "Databases/Rf/export/Facade/Rf_Arc.h"
#include "XCad/export/XCad_Cad.h"
#include "JDBHelpers/XCad/export/RMed/XCad_RMed.h"
#include "Xdm/export/xdm.h"
#include "XOS/Portability/export/Port_Type.h"

#include "main.h"

#include "CadFile.h"
#include "GridMap.h"
#include "RevArc.h"
#include "ArcCost.h"
#include "Classifier.h"
#include "DeviceModel.h"

class Tile;

// Class to hold data related to the part and the design.  Relies on
// ArgParser to initialize Data::groupData.
class Data {
public:
   typedef std::string IsolationGroupName;

   typedef std::multimap<IsolationGroupName, boost::shared_ptr<CadFile> >
     GroupData;
   typedef GroupData::iterator GroupIt;

   typedef std::set<IsolationGroupName> GroupSet;
   typedef GroupSet::iterator GroupSetIt;

   typedef std::string AreaGroupName;
   typedef std::set<AreaGroupName> AreaGroupSet;
   typedef std::multimap<AreaGroupName, IsolationGroupName> ConstraintMap;
   typedef ConstraintMap::iterator ConstraintMapIt;
   typedef std::map<IsolationGroupName, DeviceModel::SiteRectVec>
      SiteRectMap;
   typedef SiteRectMap::iterator SiteRectMapIt;
   
   typedef std::set<XCad_CadNetwork*> NetSet;
   typedef NetSet::iterator NetSetIt;

   class NetMatch {
   public:
      NetMatch(const XCad_CadNetwork* target) : b(target) {}
      bool operator()(const XCad_CadNetwork* a) const
         {
            if (a == b) return true;
            const char* aKey = a->GetName();
            const char* bKey = b->GetName();
            return strcmp(aKey, bKey) == 0;
         }
   private:
      const XCad_CadNetwork* b;
   };

   typedef
   std::map<XCad_CadNetwork*,
            IsolationGroupName,
            Classifier::Less<XCad_CadNetwork*> > NetGroup;
   typedef NetGroup::iterator NetGroupIt;

   typedef std::vector<RevArc> ArcPath;
   typedef ArcPath::iterator ArcPathIt;

   typedef std::pair<XCad_CadNetwork*, XCad_CadNetwork*> NetPair;
   class NetPairLess {
   public:
      NetPairLess(NetGroup& netGroup) : netGroup(netGroup) {}
      bool operator()(const NetPair& a, const NetPair& b) const
         {
            // Sort first by first group.
            IsolationGroupName ag1(netGroup.find(a.first)->second);
            IsolationGroupName bg1(netGroup.find(b.first)->second);
            if (ag1 < bg1) return true;
            if (bg1 < ag1) return false;

            // Sort second by second group.
            IsolationGroupName ag2(netGroup.find(a.second)->second);
            IsolationGroupName bg2(netGroup.find(b.second)->second);
            if (ag2 < bg2) return true;
            if (bg2 < ag2) return false;

            // Sort third by net name.
            Classifier::Less<XCad_CadNetwork*> less;
            return less(a.first, b.first)
               || (!less(b.first, a.first) && less(a.second, b.second));
         }
   private:
      NetGroup& netGroup;
   };

   typedef struct GroupConflict {
      XCad_CadNetwork* net1;
      IsolationGroupName group1;
      std::string fileName1;
      XCad_CadNetwork* net2;
      IsolationGroupName group2;
      std::string fileName2;
   } GroupConflict;
   typedef std::vector<GroupConflict> GroupConflictList;
   typedef GroupConflictList::iterator GroupConflictListIt;

   class GroupConflictLess {
   public:
      bool operator()(const GroupConflict& a,const GroupConflict& b) const
         {
            if (a.group1 < b.group1) return true;
            if (b.group1 < a.group1) return false;
            if (a.group2 < b.group2) return true;
            if (b.group2 < a.group2) return false;

            // Sort by net name.
            Classifier::Less<XCad_CadNetwork*> less;
            if (less(a.net1, b.net1)) return true;
            if (less(b.net1, a.net1)) return false;
            if (less(a.net2, b.net2)) return true;
            if (less(b.net2, a.net2)) return false;

            return false;
         };
   };

   typedef std::string PinName;
   // N.B. not PinGroup
   typedef std::map<PinName, IsolationGroupName> PinGroupMap;
   typedef PinGroupMap::iterator PinGroupMapIt;

   typedef std::string WhyIgnore;
   typedef std::map<WhyIgnore, Classifier::NetNameSet> WhyIgnoreMap;
   typedef WhyIgnoreMap::iterator WhyIgnoreIt;
   typedef std::set<std::pair<XCad_CadPin*, XCad_CadNetwork*> > PinNetCache;
   typedef std::vector<const XCad_CadPin*> CadPins;

public:
   Data() :
      faultsMax(FAULTS_DEFAULT),
      unroutedCount(0),
      omittedPS7(false),
      irsMultiLoadCount(0),
      deviceModel(0),
      spacingMin(SPACING_DEFAULT)
      {}

   ~Data();

public:
   DeviceModel* GetDeviceModel();
   bool NeedConstraintsCheck() const;
   void AddRange(DeviceModel::SiteRect& siteRect);
   void DeduplicateRanges();
   void AddSite(const std::string& areaGroupName, const std::string& site);
   void CollectAreaRangesFromNCD();
   void ReportSiteRanges();
   IsolationGroupName GetIsolationGroup(const XCad_CadBlock* block);
   const std::string DescribeNetConnections(XCad_CadNetwork* net);
   void ReportNetConnections(XCad_CadNetwork* net);
   void ReportBusMacroUsage();
   void ReportUserGlobalNets();
   void ReportUnusedNets();

   void CollectBusMacros();

   // Label nets with their corresponding group. Label the "combined"
   // nets first because they define the universe of discourse,
   // i.e. every other group must be a subset of the "combined"
   // group. Define the "ignored" nets next because they must all be
   // included in the "combined" nets and they enable us to ignore
   // apparent conflicts in the remaining nets. If the "ignored" net
   // list is not empty, then, verify that the "ignored" list and the
   // other groups are consistent by counting the combined nets. In the
   // end there should be no combined nets remaining.
   void LabelNets();

   void ReportSet(const Classifier::NameSet& nameSet,
                  const std::string emptyMessage);

   int GetErrorCount();
   int GetStatus();

public:                         // Net Route Checking (Virtex-4 only
                                // NCD mode) and Tile (UCF and NCD) Checking
   GroupData groupData;
   int faultsMax;
   NetGroup netGroup;
   WhyIgnoreMap whyIgnoreMap;   // Maps reason text to set of ignored nets.
   GroupConflictList groupConflictList; // Nets in multiple isolation groups.
   long unroutedCount;
   bool omittedPS7;
   long irsMultiLoadCount;      // R#check_irs_loads
   DeviceModel* deviceModel;

public:                         // UCF and NCD Area Range Checking
   SiteRectMap siteRectMap;
   Classifier classifier;

public:                         // FPGA Editor script analysis
   std::vector<std::string> scriptWires;

public:                         // constraint File (UCF) Checking
   int spacingMin;
   std::string architecture;
   std::string device;
   std::string package;
   std::string constraintsFile;
   AreaGroupSet unusedAreaGroupSet;
   ConstraintMap constraintMap;
   PinGroupMap pinGroupMap;
   std::string drawingTitle;

private:

   // Get the file name associated with a net. This is O(n) where n
   // is the total number of nets in the given group, that is to say,
   // slow, but only called for diagnostics on bad user input.
   const std::string GetFileName(XCad_CadNetwork* net,
                                 IsolationGroupName& group);

   void AddDiscardedNet(XCad_CadNetwork* net,
                        XCad_RMed* rmed,
                        const std::string& fileName);

   // The previous group for this user net is defined but it is
   // not "ignored" or "combined" thus it must be a user group.
   void AddGroupConflict(XCad_CadNetwork* net,
                         const IsolationGroupName& group,
                         const std::string& fileName);

   // If there are left over "combined" nets after inserting the other
   // nets and there are ignored nets, report the combined nets as
   // ambiguous. Unless we are in verbose mode, limit the output to ten
   // net names.
   void CheckCombinedCount(int combinedCount);

   // If a net appears in only a single isolation group, move it to
   // the ignored group if it cannot cause an isolation violation, e.g.
   // constant, clock, etc.
   void ReclassifySingleGroupNets();

   // For every net in the given group, create a mapping from it to the
   // group. Maintain the count of "combined" entries for error checking.
   void LabelNetGroups(const IsolationGroupName group, int& combinedCount);
};

// Concatenate a set of strings into a single string with the given
// separator in between items.
const std::string ToString(const Classifier::NameSet& nameSet,
                           const std::string separator = ", ");
#endif
