// Copyright (C) 2009-2013 Xilinx, Inc.  All rights reserved
// $Id: Svg.h,v 1.9.206.5 2013/05/10 00:04:05 jcorbett Exp $
//
// Generate XML and SVG syntax.

#ifndef Iv_Svg_h
#define Iv_Svg_h

#include "main.h"               // for ToString and StringReplaceAll
#include <sstream>
#include <string>
#include <vector>
#include "XOS/Portability/export/Port_Type.h"

const std::string svgFileExtension(".svg");
const std::string htmlDocType(
   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
   "<!DOCTYPE html\n"
   "      PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n"
   "      \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
);
const std::string moveToAbs(" M");
const std::string lineToAbs(" L");
const std::string horizontalLineToAbs(" H");
const std::string verticalLineToAbs(" V");
const std::string horizontalLineToRel(" h");
const std::string verticalLineToRel(" v");
const std::string closePath(" z");
const std::string dq("\"");
const std::string nl("\n");

// Generate XML syntax.
class Xml {
public:
   template <typename T>
   const std::string QuoteSpecialCharacters(const T& text) {
      std::string ts(ToString(text));
      ts = StringReplaceAll(ts, "&",  "&amp;" ); // N.B. Do this one first.
      ts = StringReplaceAll(ts, "<",  "&lt;"  );
      ts = StringReplaceAll(ts, ">",  "&gt;"  );
      ts = StringReplaceAll(ts, "\"", "&quot;");
      ts = StringReplaceAll(ts, "'",  "&apos;");
      return ts;
   }

   // R#draw_fence - Return an XML element string, e.g.
   //    <foo bar="baz">blorp</foo>
   const std::string Elt(const std::string tag,
                         const std::string attributes,
                         const std::string body="")
   {
      if (body.empty()) {
         return "<" + tag + attributes + " />";
      } else {
         return "<" + tag + attributes + ">" + body + "</" + tag + ">";
      }
   }

   // R#draw_fence - Convert data to a string and surround it with
   // double quotes.
   template <typename T>
   const std::string QQ(const T data) {
      return dq + ToString(data) + dq;
   }

   // R#draw_fence - Return an attribute/value string, e.g.
   //    bar="baz"
   template <typename T>
   const std::string Attr(const std::string attribute, const T value) {
      return " " + attribute + "=" + QQ(value);
   }

   // R#draw_fence - Return a property/value string, e.g.
   //    bar:baz;
   template <typename T>
   const std::string Decl(const std::string property, const T value) {
      return property + ":" + ToString(value) + ";";
   }

   template <typename T>
   const std::string Comment(const T& text) {
      return "<!-- " + QuoteSpecialCharacters(text) + " -->";
   }
};

// Thin wrapper for a tiny subset of SVG. Note: inverts the default
// coordinate system so the origin is at the lower left and the
// Y-coordinate increases going upward.
class Svg : public Xml {
public:
   // R#draw_fence - Use this for creating an XHTML document that
   // embeds svg.
   const std::string DocType() {
      return htmlDocType;
   }

   // R#draw_fence
   const std::string Style(const std::string style) {
      return Attr("style", style);
   }

   // R#draw_fence
   const std::string Line(double x1, double y1,
                          double x2, double y2,
                          const std::string style=
                          "stroke-opacity:0.3;stroke:black;"
                          "stroke-width:0.05")
   {
      return Elt("line",
                 Style(style)
                 + Attr("x1", x1)
                 + Attr("y1", y1)
                 + Attr("x2", x2)
                 + Attr("y2", y2));

   }

   // R#draw_fence
   const std::string Rect(double left, double top,
                          double right, double bottom,
                          const std::string style=
                          "fill:none;stroke:black;stroke-width:.05",
                          double cornerRadius=0.0,
                          const std::string title="")
   {
      const std::string
         corner(cornerRadius == 0.0
                ? std::string("")
                : Attr("rx", cornerRadius) + Attr("ry", cornerRadius));
      ASSERT(left < right);
      ASSERT(bottom < top);
      std::string titleTag;
      if (!title.empty()) {
         titleTag = Elt("title", "", QuoteSpecialCharacters(title));
      }
      return Elt("rect",
                 Style(style)
                 + Attr("x", left)
                 + Attr("y", bottom)
                 + Attr("width", right - left)
                 + Attr("height", top - bottom)
                 + corner,
                 titleTag);
   }

   // R#draw_fence - Return a string to draw a rectangle with an X
   // connecting the corners.
   const std::string XRect(double left, double top,
                           double right, double bottom,
                           const std::string style=
                           "fill:none;"
                           "stroke-opacity:1;"
                           "stroke:black;"
                           "stroke-width:0.02",
                           const std::string title="")
   {
      return
         Line(left,  top, right, bottom, style) + nl
         + Line(right, top, left,  bottom, style) + nl
         + Rect(left,  top, right, bottom, style, 0.0, title) + nl;
   }

   // Return a 'd' attribute as used in path and polygon elements.
   const std::string PathData(const std::string data) {
      return Attr("d", data);
   }

   // Boundary contains an alternating list of horizontal and vertical
   // absolute coordinates. This routine returns a string to render the
   // closed polygon they form.
   const std::string Polygon(const std::vector<double>& boundary,
                             const std::string style=
                             "fill:black;"
                             "fill-opacity:0.5;"
                             "stroke:blue;"
                             "stroke-width:.05;"
                             "stroke-opacity:.25;",
                             const std::string title="")
   {
      if (!boundary.empty()) {
         const std::string x0(ToString(boundary.at(0)));
         const std::string y0(ToString(boundary.at(1)));
         std::string d(moveToAbs + x0 + "," + y0);
         for (size_t i(2); i < boundary.size(); i += 2) {
            d += lineToAbs
               + ToString(boundary.at(i)) + "," + ToString(boundary.at(i + 1));
         }
         d += closePath;
         return Elt("path", Style(style) + PathData(d),
                    Elt("title", "", title));
      } else {
         return "";
      }
   }

   // R#draw_fence - Emit a text string.
   // Note the 'text' can be any type convertable to a string by an
   // std::ostringstream.
   template <typename T>
   const std::string Text(double x, double y,
                          const T& text,
                          const std::string style="")
   {
      std::string ts(QuoteSpecialCharacters(text));
      return Elt("text", Attr("x", x) + Attr("y", y) + style, ts);
   }

   // To display SVG embedded in XHTML, the file extension must be
   // .xhtml and the htmlDocType must be used. To display simple svg
   // in a browser, the .svg extension must be used. The former works
   // in Microsoft Internet Explorer with the Adobe SVG plugin. The
   // latter works with Mozilla Firefox.
   const std::string EmitSvg(std::string svg,
                             Port::UINT32 width=800,
                             Port::UINT32 height=800)
   {
      return Elt("svg",
                 Attr("xmlns:svg", "http://www.w3.org/2000/svg") + nl
                 + Attr("xmlns", "http://www.w3.org/2000/svg") + nl
                 + Attr("xmlns:xlink", "http://www.w3.org/1999/xlink") + nl
                 + Attr("version", "1.0") + nl
                 + Attr("width", width) + nl
                 + Attr("height", height),
                 svg);
   }
};

#endif
