// Copyright (C) 2006-2013 Xilinx, Inc.  All rights reserved
// $Id: main.h,v 1.32.206.1 2013/04/18 17:43:38 jcorbett Exp $
//
// Verify that an FPGA design constructed with the Partial
// Reconfiguration flow isolates area groups even if a given number of
// faults can be precisely injected into the design while it is
// running.
//
// This program reads a UCF file or a collection of NCD files. For the
// UCF file, it analyzes the area range constraints for site ranges,
// package pin locations, and IO buffer locations. For the NCD files,
// it analyzes the package pin locations, IO buffer locations, and
// potential routes between isolated regions via depth limited search.

#ifndef Iv_main_h
#define Iv_main_h

#include <string>
#include <sstream>
#include <exception>
#include <stdexcept>

// Wrap a macro value in double quotes
#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

// Global constants

// APPNAME is defined in our makefile
#define APPNAMESTR TOSTRING(APPNAME)

#define RPT                   ".rpt"
#define FAULTS_MAX            10
#define FAULTS_DEFAULT        2
#define SPACING_MAX           10
#define SPACING_DEFAULT       1

// Macro for sanity checks strongly expected not to fail.
#define ASSERT(ASSERTION)                                   \
   if (!(ASSERTION))                                        \
      throw std::logic_error(APPNAMESTR                     \
                       "::"  __FILE__                       \
                       ":" TOSTRING(__LINE__)               \
                       ":assertion failed: "                \
                       TOSTRING(ASSERTION)                  \
                       "\nContact Xilinx for assistance.")

const std::string WordWrap(const std::string text, size_t lineLen = 72);

const std::string StringReplaceAll(const std::string& text,
                                   const std::string& target,
                                   const std::string& replacement);

bool IsPrefix(const char* prefix, const char* candidate);

void SuppressMissingMessageFileWarning();

// The libraries that load device models and load design files send
// output to stdout. These messages are generally not helpful to end
// users. Constructing an instance of this class will suppress
// messages within the scope of the instance.
class SuppressLibraryMessages {
public:
   SuppressLibraryMessages();
   ~SuppressLibraryMessages();
};

void Console(const std::string& s);
void ConsoleDot(const std::string& s=".");
void Fatal(const std::string& message);
void Fatal(unsigned n,
           const std::string& message,
           const std::string& prefix="Error:",
           const std::string& suffix="");
void FatalInternal(unsigned n, const std::string& s);
void FatalInput(unsigned n, const std::string& s);
void FatalUsage(unsigned n, const std::string& s);
void Report(const std::string& s);
unsigned GetSectionNumber();
void SetSectionNumber(unsigned n);
unsigned GetSubsectionNumber();
void SetSubsectionNumber(unsigned n);
void Section(const std::string& s);
void Subsection(const std::string& s);
void Summary(const std::string& s);
void Warn(unsigned n, const std::string& s);
void Draw(const std::string& s);

// Convert a value of any type that ostringstream can handle to a string.
template<class T> inline
std::string ToString(const T& value) {
   std::ostringstream streamOut;
   streamOut << value;
   return streamOut.str();
}

// Convert a string to any type that can be read from a stream.
template<typename T> T ToType(const std::string& s) {
   // In all cases, this function is called on the captured text from
   // a regular expression pattern match to convert a string to an
   // integer. The patterns guarantee that the string will contain
   // at least one digit and no other classes of characters, however,
   // to guard against future mistakes, assert that the string is not
   // empty.
   ASSERT(!s.empty());
   // Initialize result to the default value for its type to address
   // a compiler warning about a potentially uninitialized variable.
   T result = T();
   std::istringstream stream(s);
   stream >> result;
   return result;
}

#endif

